function [yk, tk] = explLinMehrSchritt(f, y0, t0, tE, N, a, b, anlauf)

% Angewandte Numerik 2, Wintersemester 2016/2017, 27.01.2017
% Uebungsblatt 11, Aufgabe 37: Konvergenz expliziter Mehrschrittverfahren
%
% explLinMehrSchritt
% fuehrt ein explizites lineares Mehrschritt-Verfahren durch.
%
% Geeignet fuer Systeme von Differentialgleichungen
%
% Parameter:
%   f:      function handle der Funktion y'(t) = f(t,y),
%   y0:     Anfangswert der Anfangswertaufgabe,
%   t0:     Startzeitpunkt,
%   tE:     Endzeitpunkt,
%   N:      Anzahl der Iterationsschritte,
%   a:      Koeffizientenvektor (a_0, ..., a_k) fuer linke Seite des MSV
%   b:      Koeffizientenvektor (b_0, ..., b_k-1) fuer rechte Seite
%           (Verfahrensfunktion) des linearen MSV
%   anlauf: Funktion, die die Iterationswerte fuer das Anlaufstueck
%           liefert, Signatur wie bisher bei ESV,
%
% Rueckgabewert:
%   yk:     Vektor der einzelnen Naeherungswerte an den Gitterpunkten
%   tk:     Gitter mit den diskreten Zeitpunkten, an denen eine
%           Naeherungsloesung der AWA berechnet wurde



%% Initialisierungen

[i,j] = size(a);
k = i-1;
if (j ~= 1)
    error('a hat nicht genau eine Spalte');
end

[i,j] = size(b);
if (j ~= 1)
    error('b hat nicht genau eine Spalte');
end

if (i ~= k)
    error('Koeffizientenvektoren haben falsche Laenge');
end

tk = linspace(t0, tE, N+1);
h  = tk(2) - tk(1);

dim = length(y0);         	% Dimension (Anzahl Komponenten) des Systems
yk = zeros(dim, N+1);     	% Je Iterationswert eine Spalte
fk = zeros(dim, N+1);     	% Funktionswerte f(tk(j), yk(j))



%% Anlaufstueck berechnen

yk(:,1) = y0;                   % Anfangswert eintragen in Vektor
                                % eigentlich ueberfluessig (Anlaufstueck)
yk(:,1:k) = anlauf(f, y0, tk(1:k));     % Iterationswerte des Anlaufstuecks

for j = 1:k                     % Funktionswerte des Anlaufstuecks
    fk(:,j) = f(tk(j), yk(:,j));
end



%% Iterationsschleife des expliziten Mehrschrittverfahrens

for j = 1:N+1-k                 % durchlaufe Gitterpunkte
    yk(:,j+k) = (-yk(:,j:j+k-1)*a(1:k) + h * fk(:,j:j+k-1)*b) / a(k+1);
    fk(:,j+k) = f(tk(j+k), yk(:,j+k));
end


end

