% Angewandte Numerik 2, Wintersemester 2016/2017, 27.01.2017
% Uebungsblatt 11, Aufgabe 37: Konvergenz expliziter Mehrschrittverfahren
%
% Loest eine Anfangswertaufgabe mit Zweischrittverfahren aus Aufgabe 36.
% In Aufgabe 36 haben wir gezeigt,
%   -   dass das Zweischrittverfahren der Ordnung 3 instabil ist,
%   -   dass es stabile Zweischrittverfahren der Ordnung 2 gibt.



%% Konstante und Initialisierungen

clear all;
close all;
clc;

msv = 1;                    % Teil b: MSV der Konsistenz-Ordnung 3
% msv = 2;                    % Teil f: MSV der Ordnung 2



%% Zu Aufgabenteil f und g:

%  Fuer den Fall msv = 2: alpha1 kann veraendert werden
%  - alpha1 in (-2, 0]:              Stabilitaet, Konvergenz
%  - alpha1 <= -2 oder alpha1 > 0:   Instabilitaet, keine Konvergenz

    alpha1 =  0.10;
    alpha1 =  0.01;
    alpha1 =  0.00;
    alpha1 = -0.01;
    alpha1 = -1.00;
 	alpha1 = -1.80;
 	alpha1 = -1.90;
 	alpha1 = -1.99;
  	alpha1 = -2.00;
  	alpha1 = -2.01;



%% Festlegung der Anzahl der Gitter-Punkte der Diskretisierung

nMax = 0;                           % Aufgabenteil b)
% nMax = 3;                           % Aufgabenteil d)
% nMax = 8;                           % Aufgabenteil f)
% nMax = 12;          % Achtung: nicht fuer alpha1 > 0

N = 10 * 2.^(0:nMax);               % Anzahl Gitterpunkte



%% Die Anfangswertaufgabe

f = @(t,y) y;
t0 = 0;                             % Anfangszeitpunkt
tEnd = 1;                           % Endzeitpunkt

yExakt = @(t) exp(t);               % die exakte Loesung



%% Verschiedene Mehrschrittverfahren

switch msv
    case 1                          % instabil, Konsistenz-Ordnung 3
        alpha1 = 4;                 % in Aufgabe 36 Teil c) berechnet
        
        alpha0 = - 1 - alpha1;      % analog Aufgabe 36 Teil a)
        beta0  = alpha1 / 2;
        beta1  = 2 + alpha1 / 2;

        titleStr = 'Instabiles Zweischrittverfahren';
        a = [alpha0; alpha1; 1];    % Koeffizientenvektor (a0, a1, a2)
        b = [beta0;  beta1];        % Koeffizientenvektor (b0, b1)
    case 2                          % stabil, Konsistenz-Ordnung 2
        alpha0 = - 1 - alpha1;      % analog Aufgabe 36 Teil a)
        beta0  = alpha1 / 2;
        beta1  = 2 + alpha1 / 2;

        titleStr = 'Stabiles Zweischrittverfahren';
        a = [alpha0; alpha1; 1];    % Koeffizientenvektor (a0, a1, a2)
        b = [beta0;  beta1];        % Koeffizientenvektor (b0, b1)
    case 3                          % Numerik 4, SoSe 2012, Aufgabe 26
        titleStr = 'Dreischrittverfahren aus SoSe 2012';
        a = [ 4; -1; -4;  1];
        b = [-1; -6;  1];
    otherwise
        error('Beispiel %d gibt es nicht', bsp);
end



%% Speicherallokation und Initialisierungen

err = zeros(1,length(N));	% maximaler globaler Diskretisierungsfehler
                            % fuer alle Gitter-Feinheiten

fprintf('\n');
fprintf('\n');
fprintf('Aufgabe 37: Konvergenz von expliziten Mehrschrittverfahren\n');
fprintf('\n');



%% Schleife ueber alle zu untersuchenden Gitter-Feinheiten

for i=1:length(N)
    fprintf('Schrittweite = 1/%g\n', N(i));
        
    [yk, tk] = explLinMehrSchritt(f, yExakt(t0), t0, tEnd, N(i), a, b, ...
                            @(f, yStart, tk) yExakt(tk));                   
    err(i) = max(max(abs(yk - yExakt(tk))));    % Unendlich-Norm
    
    
    
    %% Grafische Ausgabe
    
    figure(i);                  % jede Schrittweite in ein neues Schaubild 
    plot(tk, yk, 'x-')
    titleStr = sprintf('%s mit Schrittweite 1/%i', titleStr, N(i));
    title(titleStr, 'FontSize', 14)
    xlabel('t', 'FontSize', 14)
    ylabel('y', 'FontSize', 14)
    xlim ([0,1]);
    ylim ([0,5]);
    hold on;
    plot(tk, yExakt(tk), 'r')
    legend ('Numerische Loesung', 'Exakte Loesung', 'Location', 'Northwest')
end



%% Berechnung und Ausgabe der numerischen Konvergenzordnungen

fprintf('\n\n');
fprintf('Numerische Konvergenzordnungen:\n');
fprintf('\n');
fprintf('Anzahl Gitterpkte | Fehler                        | Numerische\n');
fprintf('   grob |    fein | grobes Gitter | feines Gitter | KonvergenzOrdn.\n');
fprintf('--------+---------+---------------+---------------+----------------\n');

for i = 1:length(N)-1
    fprintf('%7d | %7d | %13e | %13e | %13.9f\n', ...
            N(i), N(i+1), err(i), err(i+1), ...
            log(err(i)/err(i+1))/log(2));
end
