% Angewandte Numerik 2, Wintersemester 2016/2017, 27.01.2017
% Uebungsblatt 11, Aufgabe 38: Einfluss des ESV fuer das Anlaufstueck
%
% Untersucht, wie sich das zur Berechnung der Werte des Anlaufstuecks
% verwendete ESV auf die Konvergenzordnung des MSV auswirkt



%% Konstante und Initialisierungen

clear all;
close all;
clc;

awa = 'a';          % Anfangswertaufgabe aus Aufgabenteil a)
% awa = 'b';          % Anfangswertaufgabe aus Aufgabenteil b)
% awa = 'c';          % Anfangswertaufgabe aus Aufgabenteil c)



%% Festlegung der Anzahl der Gitter-Punkte der Diskretisierung

nMax = 9;          	% Analog zur Aufgabenstellung

N = 10 * 2.^[0:nMax];               % Anzahl Gitterpunkte



%% Die Anfangswertaufgaben

switch awa
    case 'a'                        % Aufgabe 38, Teil a)
        f      = @(t,y) y;
        yExakt = @(t) exp(t);       % die exakte Loesung
        t0     = 0;                 % Anfangszeitpunkt
        tEnd   = 1;                 % Endzeitpunkt
        c      = 10;                % Faktor fuer die Steigungsgeraden
    case 'b'                        % Aufgabe 38, Teil b)
        f      = @(t,y) -2 .* t .* y.^2;
        yExakt = @(t) 1./(t.^2+1);  % die exakte Loesung
        t0     = 0;                 % Anfangszeitpunkt
        tEnd   = 1;                 % Endzeitpunkt
        c      = 20;                % Faktor fuer die Steigungsgeraden
    case 'c'                        % Aufgabe 38, Teil c)
        f      = @(t,y) [            y(2) - y(3);
                         -2*y(1) + 3*y(2) - y(3);
                         -  y(1) +   y(2) + y(3)];
        yExakt = @(t) [   exp(t) - 4*t.*exp(t);
                          exp(t) - 4*t.*exp(t) - 2*exp(2*t);
                       4.*exp(t)               - 2*exp(2*t)];
        t0     = 0;                 % Anfangszeitpunkt
        tEnd   = 2;                 % Endzeitpunkt
        c      = 50000;             % Faktor fuer die Steigungsgeraden
    otherwise
        error('Anfangswertaufgabe %d gibt es nicht', awa);
end



%% Das Mehrschrittverfahren: Adams-Bashforth-Verfahren

titleStr = 'Adams-Bashforth-Verfahren';
a = [ 0;  0;  0; -1;  1];   % Koeffizientenvektor linke Seite
b = [-9; 37;-59; 55] / 24;  % Koeffizientenvektor Verfahrensfunktion



%% Speicherallokation und Initialisierungen

err = zeros(4,size(N,2));       	% 4 verschiedene Anlaufvarianten

fprintf('\n');
fprintf('\n');
fprintf('Aufgabe 38: Einfluss des fuer das Anlaufstueck verwendeten ESV\n');
fprintf('\n');



%% Schleife ueber alle zu untersuchenden Gitter-Feinheiten

for i=1:length(N)
    fprintf('Schrittweite = 1/%g\n', N(i));
        
    [yk, tk] = explLinMehrSchritt(f, yExakt(t0), t0, tEnd, N(i), a, b, ...
               	@(f, yStart, tk) yExakt(tk));
    err(1,i) = max(max(abs(yk - yExakt(tk))));
    
    [yk, tk] = explLinMehrSchritt(f, yExakt(t0), t0, tEnd, N(i), a, b, ...
              	@eulerExplizit);
    err(2,i) = max(max(abs(yk - yExakt(tk))));
    
    [yk, tk] = explLinMehrSchritt(f, yExakt(t0), t0, tEnd, N(i), a, b, ...
              	@(f, yStart, tk) rungeKutta(f,yStart,tk,rKverbEuler));
    err(3,i) = max(max(abs(yk - yExakt(tk))));
    
    [yk, tk] = explLinMehrSchritt(f, yExakt(t0), t0, tEnd, N(i), a, b, ...
               	@(f, yStart, tk) rungeKutta(f,yStart,tk,rK4));
    err(4,i) = max(max(abs(yk - yExakt(tk))));
end



%% Graphische Ausgabe

fprintf('\n');
loglog(N, err(1,:), '-*', N, err(2,:), '-o', ...
       N, err(3,:), '-s', N, err(4,:), '--d')
title('Vergleich verschiedener ESV zur Berechnung des Anlaufstuecks', ...
      'FontSize', 14);
xlabel(sprintf('Anzahl der Gitterpunkte im Zeitintervall [%g, %g]', t0, tEnd));
ylabel('maximaler Fehler');

hold on                         % Steigungsgeraden einzeichnen
xStg = [N(2), N(9)];
loglog(xStg, c*1.0*xStg.^-1,  '-k', xStg, c*0.2*xStg.^-2, '-.k', ...
       xStg, c*0.08*xStg.^-3, ':k', xStg, c*0.08*xStg.^-4, '-k')

legend('exakte Werte', ...
       'explizites Eulerverfahren', ...
       'verbessertes Euler-Verfahren', ...
       'klassisches Runge-Kutta-Verfahren 4. Ordnung', ...
       'Steigung -1', 'Steigung -2', 'Steigung -3', 'Steigung -4');
