function yk = rungeKutta(f, y0, tk, bt)

% Angewandte Numerik 2, Wintersemester 2016/2017, 20.01.2017
% Uebungsblatt 10, Aufgabe 31: Weitere Einschrittverfahren
%
% rungeKutta berechnet die Loesung einer Anfangswertaufgabe
% y' = f(t,y), y(t0) = y0 mit einem expliziten Runge-Kutta-Verfahren.
% Implementierung ist unabhaengig von konkreten Butcher-Tableau.
%
% Parameter:
%   f:      function handle der Funktion f(t,y)
%   y0:     Anfangswert der Anfangswertaufgabe
%   tk:     Gitter mit den diskreten Zeitpunkten, an denen eine
%           Naeherungsloesung der AWA berechnet werden soll
%   bt:     Butcher-Tableau als Struktur mit den Komponenten
%           - bt.m:     die Stufenzahl des Runge-Kutta-Verfahrens
%           - bt.alpha: der Spaltenvektor alpha des RK-Verfahrens
%           - bt.beta:  die Matrix beta des RK-Verfahrens
%           - bt.gamma: der Zeilenvektor beta des RK-Verfahrens
%
% Rueckgabewert:
%   yk:     Vektor der einzelnen Naeherungswerte an den Gitterpunkten



%% Das Butcher-Tableau ueberpruefen

checkBt(bt);



%% Zeitschleife: Berechnung der Naeherungswerte

N = length(tk);          	% Anzahl der Auswertezeitpunkte im Gitter
dim = length(y0);         	% Dimension (Anzahl Komponenten) des Systems

yk=zeros(dim, N);
yk(:,1) = y0;            	% Anfangswert eintragen in Vektor

for k = 1:N-1             	% durchlaufe Gitterpunkte
    h = tk(k+1)-tk(k);   	% aktuelle Schrittweite
    
    ki = zeros(dim, bt.m);
    for i = 1:bt.m      	% Alle Zeilen des Butcher-Tableaus durchlaufen
        ki(:,i) = f( tk(k) + bt.alpha(i)*h, ...
                     yk(:,k) + h * (ki(:,1:i-1) * bt.beta(i,1:i-1)') );
    end
    
                            % Verfahrensfunktion des
                            % Runge-Kutta-Verfahrens anwenden
    yk(:,k+1) = yk(:,k) + h*(ki*bt.gamma');
end

end